﻿using FLY.FilmCasting.AutoDie.IService;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Threading;

namespace FLY.FilmCasting.AutoDie.Server
{
    /// <summary>
    /// PlcLink 只有 1~100， 最小单位1，  扩展它， 1~100， 最小单位 0.1
    /// </summary>
    public class PlcLinkExt: IPlcLinkExt
    {
        /// <summary>
        /// 每颗粒时间 5s
        /// </summary>
        const int CellSec = 3;

        /// <summary>
        /// 占空比，总颗粒数
        /// </summary>
        const int DutyCount = 10;

        int channelCnt = 96;
        double[] heats;


        /// <summary>
        /// 当前电流 有没？
        /// </summary>
        public bool HasElectricity { get; private set; }
        /// <summary>
        /// 风机是否启动？
        /// </summary>
        public bool HasFan { get; private set; }

        public bool IsConected { get; private set; }

        public event PropertyChangedEventHandler PropertyChanged;

        PLCLink plcLink;
        DispatcherTimer timer;
        Stopwatch stopwatch;
        int dutyIndex = 0;
        public PlcLinkExt() 
        {
        
        }
        public void Init(PLCLink plcLink) 
        {
            this.plcLink = plcLink;

            Misc.BindingOperations.SetBinding(plcLink, nameof(plcLink.IsConected), this, nameof(IsConected));
            Misc.BindingOperations.SetBinding(plcLink, nameof(plcLink.HasElectricity), this, nameof(HasElectricity));
            Misc.BindingOperations.SetBinding(plcLink, nameof(plcLink.HasFan), this, nameof(HasFan));

            

            this.PropertyChanged += PlcLinkExt_PropertyChanged;
            timer = new DispatcherTimer();
            timer.Interval = TimeSpan.FromSeconds(1);
            timer.Tick += Timer_Tick;

            stopwatch = new Stopwatch();

            if (plcLink.IsConected)
            {
                //刚连上，把 之前的数据都重新发给它。
                plcLink.SetChannelCnt((UInt16)channelCnt);
                DutyStart();
            }
        }

        void DutyStart() 
        {
            stopwatch.Restart();
            timer.Start();
            dutyIndex = 0;
            setHeats();
        }
        void DutyStop() 
        {
            stopwatch.Stop();
            timer.Stop();
        }
        void setHeats() 
        {
            if (!plcLink.IsConected)
                return;

            UInt16[] plcHeats = heats.Select(h => {
                int h_int = (int)h;
                int v = (int)((h - h_int) * DutyCount);
                if (v > dutyIndex)
                {
                    return (UInt16)(h_int + 1); 
                }
                else {
                    return (UInt16)h_int;
                }
                
            }) as UInt16[];
            plcLink.SetHeat(plcHeats);

            UInt16 heatupdate = (UInt16)(plcLink.HeatUpdate_R + 1);
            if (heatupdate > 100)
                heatupdate = 1;
            plcLink.SetHeatUpdate(heatupdate);
        }

        private void Timer_Tick(object sender, EventArgs e)
        {
            if (stopwatch.Elapsed < TimeSpan.FromSeconds(CellSec)) 
            {
                return;
            }

            stopwatch.Restart();

            setHeats();
            dutyIndex++;
            if (dutyIndex > DutyCount)
                dutyIndex = 0;
        }

        private void PlcLinkExt_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == nameof(plcLink.IsConected)) 
            {
                if (plcLink.IsConected)
                {
                    //刚连上，把 之前的数据都重新发给它。
                    plcLink.SetChannelCnt((UInt16)channelCnt);
                    DutyStart();
                }
                else {
                    DutyStop();
                }
            }
        }

        public void SetChannelCnt(int channelCnt) 
        {
            this.channelCnt = channelCnt;
            plcLink.SetChannelCnt((UInt16)channelCnt);
        }

        /// <summary>
        /// 设置加热量
        /// </summary>
        /// <param name="values"></param>
        public void SetHeats(IEnumerable<double> values) 
        {
            heats = values.Select(h =>
            {
                if (h < 0) h = 0;
                else if (h > 100) h = 100;
                return h;
            }).ToArray();

            setHeats();
        }


    }

}
