﻿using SQLite;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FLY.Weight.Server.Model
{
    /// <summary>
    /// 流量表
    /// </summary>
    [Table("Flow")]
    public class Db_Flow: IDbBase
    {
        [Key]
        [Index]
        [PropertyIndex(0)]
        public Int64 ID { get; set; }

        /// <summary>
        /// 开始时间
        /// </summary>
        [Time]
        [Index]
        [PropertyIndex(1)]
        public DateTime Time { get; set; }

        /// <summary>
        /// 总流量
        /// </summary>
        [PropertyIndex(2)]
        public double Total { get; set; }


        /// <summary>
        /// 各上料流量 List of FlyData_FlowItem
        /// </summary>
        [PropertyIndex(3)]
        public string Items { get; set; }
    }

    /// <summary>
    /// 混料表
    /// </summary>
    [Table("Mix")]
    public class Db_Mix: IDbBase
    {
        [Key]
        [Index]
        [PropertyIndex(0)]
        public Int64 ID { get; set; }

        /// <summary>
        /// 层序号 0~5,  每层标识
        /// </summary>
        [Index]
        [PropertyIndex(1)]
        public int Idx { get; set; }

        /// <summary>
        /// 开始时间
        /// </summary>
        [Time]
        [Index]
        [PropertyIndex(2)]
        public DateTime Time { get; set; }

        /// <summary>
        /// 料斗重量
        /// </summary>
        [PropertyIndex(3)]
        public double Total { get; set; }


        /// <summary>
        /// 各上料流量 List of FlyData_MixItem
        /// </summary>
        [PropertyIndex(3)]
        public string Items { get; set; }
    }

    /// <summary>
    /// 混料表
    /// </summary>
    [Table("RollMix")]
    public class Db_RollMix : IDbBase
    {
        [Key]
        [Index]
        [PropertyIndex(0)]
        public Int64 ID { get; set; }

        /// <summary>
        /// 层序号 0~5,  每层标识
        /// </summary>
        [Index]
        [PropertyIndex(1)]
        public int Idx { get; set; }

        /// <summary>
        /// 开始时间
        /// </summary>
        [Time]
        [Index]
        [PropertyIndex(2)]
        public DateTime Time { get; set; }

        /// <summary>
        /// 料斗重量
        /// </summary>
        [PropertyIndex(3)]
        public double Total { get; set; }


        /// <summary>
        /// 各上料流量 List of FlyData_MixItem
        /// </summary>
        [PropertyIndex(3)]
        public string Items { get; set; }
    }
}
