﻿using FLY.OBJComponents.Server;
using FLY.OBJComponents.Server.OBJProxy;
using FLY.Weight.Server.Model;
using FLY.Weight.Server.OBJProxy;
using FObjBase;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;

namespace FLY.Weight.Server
{
    public class TDGage
    {
        #region 初始化------------------------------------------------------------------
        public string mParamDirectory;//参数目录 m_path;//数据文件的 根目录默认是 Gage1
        public string mName;//名字 如:gage0
        #endregion

        #region 本地数据----------------------------------------------------------------
        public SysParam mSysParam;
        public WeightSystem mData;
        public HistoryDb mHistoryDb;
        /// <summary>
        /// 报警系统
        /// </summary>
        public WarningSystem2 mWarning;

        DbModel mDbModel;
        #endregion

        #region 服务器数据--------------------------------------------------------------

        FLY.Thick.Base.Client.TDGageServiceClient tDGageServiceClient;
        
        #endregion

        public TDGage(string nam)
        {
            mName = nam;
            mParamDirectory = mName;// @"D:\" + m_name;

            if (!System.IO.Directory.Exists(mParamDirectory))
            {
                System.IO.Directory.CreateDirectory(mParamDirectory);
            }
            System.Environment.CurrentDirectory = mParamDirectory;

            mSysParam = new SysParam();

            mData = new WeightSystem();

            mDbModel = new DbModel();
            mHistoryDb = new HistoryDb();
            mWarning = new WarningSystem2();

        }
        /// <summary>
        /// 第1步, 加载本地数据
        /// </summary>
        public async Task Init()
        {
            //---------------------------------------------------------------------------------------------------------------
            //数据库
            await Task.Factory.StartNew(() =>
            {
                //初始数据库(加载 MaxId)
                mDbModel.Init();

                //删除不需要的备份数据
                mDbModel.KeepBackupSize(12);

                //备份上一个月之前的数据到 yyyy-MM 文件夹
                mDbModel.BackupBbInSize(12);

                //当前的数据库只保存上个月与这个月的数据
                mDbModel.KeepDBSize(1);
            });
            
            mHistoryDb.Init(mDbModel, mData.ItemsCnt);

            mWarning.Init(mHistoryDb.ErrorBuffer);

            mData.Init(mHistoryDb, mWarning);

            string connName = "thick";
            #region 与服务器同步的数据
            FObjServiceClientManager.Instance.ConnAddrs.Add(new ConnAddr() { ConnName = connName, Addr = mSysParam.BlowingAddr });

            tDGageServiceClient = new Thick.Base.Client.TDGageServiceClient(FLY.Thick.FilmCasting.OBJ_INTERFACE.OBJ_INTERFACE_ID.TDGAGE_ID, connName);
            #endregion


            tDGageServiceClient.DynArea.PropertyChanged += DynArea_PropertyChanged;

        }

        private void DynArea_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            if (e.PropertyName == nameof(Thick.Base.Common.DynArea.Width))
            {
                double v = Math.Round(tDGageServiceClient.DynArea.Width / 10.0);
                v = Math.Round(v * 10);

                mData.Accessory.TotalFilmWidth = (float)v;
            }
            else if (e.PropertyName == nameof(Thick.Base.Common.DynArea.FilmVelocity)) {

                double v = Math.Round(tDGageServiceClient.DynArea.FilmVelocity);
                mData.Accessory.CurrentVelocitySet = (float)v;
            }
        }      
    }
}
